### R code from vignette source 'opm-substrates.Rnw'

###################################################
### code chunk number 1: chunk-setup
###################################################
options(width = 90)
options(prompt = "R> ")
options(continue = "   ")
options(useFancyQuotes = FALSE)
library("methods")


###################################################
### code chunk number 2: load-library
###################################################
suppressPackageStartupMessages(library("pathview"))
if ("package:opm" %in% search())
  detach("package:opm", unload = TRUE)
library("opm")
data(vaas_et_al, package = "opmdata")


###################################################
### code chunk number 3: plate-types
###################################################
plate_type(full = TRUE)


###################################################
### code chunk number 4: find-substrate-exact-substring
###################################################
substrates <- find_substrate(c("Glutamine", "Glutamic acid"))
substrates


###################################################
### code chunk number 5: find-substrate-globbing-wrong
###################################################
substrates <- find_substrate(c("L-Glutamine", "L-Glutamic acid"), "glob")
substrates


###################################################
### code chunk number 6: find-substrate-globbing-useful
###################################################
substrates <- find_substrate(c("*L-Glutamine", "*L-Glutamic acid"), "glob")
substrates


###################################################
### code chunk number 7: find-positions
###################################################
positions <- find_positions(substrates)
positions


###################################################
### code chunk number 8: substrate-info
###################################################
subst.info <- substrate_info(substrates)
subst.info


###################################################
### code chunk number 9: substrate-all-info
###################################################
subst.info <- substrate_info(substrates, "all")
subst.info


###################################################
### code chunk number 10: annotated
###################################################
x <- annotated(vaas_1)
head(x)


###################################################
### code chunk number 11: annotated-param
###################################################
x <- annotated(vaas_1, output = param_names()[4])
head(x)


###################################################
### code chunk number 12: annotated-opm_glht-plot
###################################################
x <- opm_mcp(vaas_4[, , 1:15], output = "mcp", model = ~ Well,
  linfct = c(Dunnett.A05 = 1), full = FALSE)


###################################################
### code chunk number 13: annotated-opm-glht-figure
###################################################
old.mar <- par(mar = c(3, 8, 3, 2)) # adapt plotting settings
plot(x)
par(old.mar) # reset plotting settings


###################################################
### code chunk number 14: annotated-opm_glht-options
###################################################
annotab <- c("numeric", "upwards", "downwards", "different", "equal",
  "smaller", "larger")
annotab <- mapply(output = annotab, annotated,
  MoreArgs = list(object = x), SIMPLIFY = FALSE)
annotab <- data.frame(Comparison = rownames(confint(x)$confint), annotab)
annotab


###################################################
### code chunk number 15: example-data
###################################################
coli.sub <- subset(vaas_et_al, list(Species = "Escherichia coli",
  Experiment = "First replicate"))
coli.k12 <- subset(coli.sub, list(Strain = "DSM18039"))
coli.type <- subset(coli.sub, list(Strain = "DSM30083T"))


###################################################
### code chunk number 16: example-data-anno
###################################################
anno.k12 <- annotated(coli.k12)
anno.type <- annotated(coli.type)


###################################################
### code chunk number 17: pathview-wrapper
###################################################
opm_path <- function(cpd.data, gene.data = NULL,
    high = list(gene = "green4", cpd = "blue"),
    mid = list(gene = "lightsteelblue1", cpd = "yellow"),
    low = list(gene = "white", cpd = "yellow"),
    species = "ko", out.suffix = "non-native",
    key.pos = "topright", afactor = 1000,
    limit = list(gene = 2, cpd = 400),
    bins = list(gene = 0.5, cpd = 40),
    both.dirs = list(gene = FALSE, cpd = FALSE),
    sign.pos = "topleft", cpd.lab.offset = 0,
    same.layer = FALSE,
    na.col = "white", ...) {
  pathview(cpd.data = cpd.data, gene.data = gene.data,
    high = high, mid = mid, low = low,
    species = species, out.suffix = out.suffix, key.pos = key.pos,
    afactor = afactor, limit = limit, bins = bins,
    both.dirs = both.dirs, sign.pos = sign.pos,
    cpd.lab.offset = cpd.lab.offset, same.layer = same.layer,
    na.col = na.col, ...)
}


###################################################
### code chunk number 18: pathview-wrapper-plot-native
###################################################
coli.map.k12 <- opm_path(cpd.data = anno.k12, species = "ko",
  out.suffix = "k12.ko", pathway.id = "00052")
coli.map.type <- opm_path(cpd.data = anno.type, species = "ko",
  out.suffix = "type.ko", pathway.id = "00052")


###################################################
### code chunk number 19: pathview-wrapper-plot-bin
###################################################
anno.k12.bin <- annotated(coli.k12, output = param_names("disc.name"),
  lmap = 1:3)
anno.type.bin <- annotated(coli.type, output = param_names("disc.name"),
  lmap = 1:3)
coli.map.k12.bin <- opm_path(cpd.data = anno.k12.bin, species = "ko",
  out.suffix = "k12.ko.bin", pathway.id = "00052",
  limit = list(gene = 2, cpd = c(1, 3)), bins = list(gene = 0.5, cpd = 3))
coli.map.type.bin <- opm_path(cpd.data = anno.type.bin, species = "ko",
  out.suffix = "type.ko.bin", pathway.id = "00052",
  limit = list(gene = 2, cpd = c(1, 3)), bins = list(gene = 0.5, cpd = 3))


###################################################
### code chunk number 20: pathview_k12_comp_map (eval = FALSE)
###################################################
## coli.map.k12.eco <- opm_path(cpd.data = anno.k12, species = "eco",
##   out.suffix = "k12.eco", pathway.id = "00052", na.col = "pink")


###################################################
### code chunk number 21: anno_sum_mat_col
###################################################
anno.k12.mat <- annotated(coli.k12, how = "value")
col.sums <- sort(colSums(anno.k12.mat, na.rm = TRUE), decreasing = TRUE)
col.sums[1:10]


###################################################
### code chunk number 22: e-coli-sub-pos
###################################################
e.subs <- rownames(anno.k12.mat)[!is.na(anno.k12.mat[, "map00052"]) &
  anno.k12.mat[, "map00052"] > 0]
e.subs.pos <- find_positions(e.subs, type = "Gen III")
e.subs.pos


###################################################
### code chunk number 23: e-coli-single-ci-figure-code (eval = FALSE)
###################################################
## old.mar <- par(mar = c(3, 2, 2, 2)) # adapt plotting settings
## ci_plot(object = coli.sub[, , e.subs.pos],
##   as.labels = list("Species", "Strain"), subset = "A", x = "bottomright",
##   draw.legend = TRUE, crr = 1.33, legend.field = NULL)
## par(old.mar) # reset plotting settings


###################################################
### code chunk number 24: e-coli-single-ci-figure
###################################################
old.mar <- par(mar = c(3, 2, 2, 2)) # adapt plotting settings
ci_plot(object = coli.sub[, , e.subs.pos],
  as.labels = list("Species", "Strain"), subset = "A", x = "bottomright",
  draw.legend = TRUE, crr = 1.33, legend.field = NULL)
par(old.mar) # reset plotting settings


###################################################
### code chunk number 25: pathview-e_coli-comp
###################################################
coli.comp <- opm_mcp(coli.sub[, , e.subs.pos],
  output = "mcp", model = ~ J(Well + Strain), linfct = c(Pairs = 1))


###################################################
### code chunk number 26: e-coli-comp-ci-figure
###################################################
old.mar <- par(mar = c(3, 20, 3, 0.5)) # adapt plotting settings
plot(coli.comp, xlim = c(-5, 290))
abline(v = 150, col = "blue", lty = 5, lwd = 2)
par(old.mar) # reset plotting settings


###################################################
### code chunk number 27: pathview_e_coli_comp_map
###################################################
coli.comp.map <- opm_path(cpd.data = annotated(coli.comp), species = "ko",
  out.suffix = "coli.comp.ko", pathway.id = "00052")


###################################################
### code chunk number 28: pathview-cat-anno-map
###################################################
# 'upwards' comparison, default minimum effect size
cat.coli.comp.0 <- opm_path(
  cpd.data = annotated(coli.comp, output = "upwards", lmap = 1:3),
  species = "ko", out.suffix = "cat-coli-comp-0", pathway.id = "00052",
  limit = list(gene = 2, cpd = c(1, 3)), bins = list(gene = 0.5, cpd = 3))
# 'upwards' comparison, 150 units as minimum effect size
cat.coli.comp.150 <- opm_path(
  cpd.data = annotated(coli.comp, output = "'150", lmap = 1:3),
  species = "ko", out.suffix = "cat-coli-comp-150", pathway.id = "00052",
  limit = list(gene = 2, cpd = c(1, 3)), bins = list(gene = 0.5, cpd = 3))


###################################################
### code chunk number 29: pathview-plot-graphviz
###################################################
coli.graphvizmap.k12 <- opm_path(cpd.data = anno.k12, species = "ko",
  afactor = 1500, kegg.native = FALSE, out.suffix = "graphvizk12.ko",
  pathway.id = "00052", sign.pos = "bottomleft", key.pos = "bottomright")


###################################################
### code chunk number 30: increase_number_of_cores
###################################################
## if possible, use 4 cores; change this if necessary
options(mc.cores = 4)


###################################################
### code chunk number 31: mboost_model_fitting
###################################################
## load package mboost
library("mboost")
## set seed (for reproducible results)
set.seed(1907)


###################################################
### code chunk number 32: make_data_set
###################################################
## use multiple biological replicates:
coli.sub1 <- subset(vaas_et_al, list(Species = "Escherichia coli",
  Experiment = c("First replicate", "Second replicate")))

## reformat and add substrate information
data <- annotated(opm_mcp(coli.sub1,
  ~ Strain + Experiment + Slot + 'Plate number',
  output = "data", in.parens = FALSE), how = "data.frame")
data <- data[complete.cases(data),
  names(data) %in% c("Plate.number", "Strain", "Experiment",
    "Slot", "Value") | grepl("^map", names(data))]

## add an ID per biological / technical replicate for usage of
## brandom (subject-specific random effect, see below):
data$ID <- extract_columns(data, c("Plate.number", "Strain", "Experiment"),
  direct = TRUE, factor = TRUE)

## now the plate number is not needed any more
data$Plate.number <- NULL

## have a look at a random subset of the resulting data set
data[sample(1:nrow(data), size = 5), 1:7]


###################################################
### code chunk number 33: set_seed
###################################################
set.seed(1907)


###################################################
### code chunk number 34: model_code (eval = FALSE)
###################################################
## ## re-define linear base-learner to have 2 degrees of freedom
## bols2 <- function(a, ...)
##   bols(a, df = 2, ...)
## 
## ## fit offset model with main effects only (and use very many iterations):
## options(contrasts = c("contr.treatment", "contr.poly"))
## offsetmod <- gamboost(log(Value) ~ ., data = data, baselearner = bols2,
##   control = boost_control(mstop = 6000, nu = 0.2))
## 
## ## now start from the offset model and add interaction effects (which
## ## represent differences between strains).
## options(contrasts = c("contr.sum", "contr.poly"))
## 
## ## set up model formula
## fm <- as.formula(paste(c("log(Value) ~ bols(Strain, intercept = FALSE, df = 1)",
##     ## add variables as main effects
##     sprintf("bols(%s, intercept = FALSE, df = 1)", names(data)[-c(1, 4)]),
##     ## add the interaction terms (i.e. strain-specific effects)
##     sprintf("bols(%s, by = Strain, intercept = FALSE, df = 1)",
##       names(data)[-c(1, 4)])),
##   collapse = " + "))
## 
## ## estimate model
## mod <- gamboost(fm, data = data, offset = fitted(offsetmod),
##   control = boost_control(mstop = 250, nu = 0.2))
## options(contrasts = c("contr.treatment", "contr.poly"))


###################################################
### code chunk number 35: model
###################################################
if (!file.exists("pathway_model.Rda")) {
## re-define linear base-learner to have 2 degrees of freedom
bols2 <- function(a, ...)
  bols(a, df = 2, ...)

## fit offset model with main effects only (and use very many iterations):
options(contrasts = c("contr.treatment", "contr.poly"))
offsetmod <- gamboost(log(Value) ~ ., data = data, baselearner = bols2,
  control = boost_control(mstop = 6000, nu = 0.2))

## now start from the offset model and add interaction effects (which
## represent differences between strains).
options(contrasts = c("contr.sum", "contr.poly"))

## set up model formula
fm <- as.formula(paste(c("log(Value) ~ bols(Strain, intercept = FALSE, df = 1)",
    ## add variables as main effects
    sprintf("bols(%s, intercept = FALSE, df = 1)", names(data)[-c(1, 4)]),
    ## add the interaction terms (i.e. strain-specific effects)
    sprintf("bols(%s, by = Strain, intercept = FALSE, df = 1)",
      names(data)[-c(1, 4)])),
  collapse = " + "))

## estimate model
mod <- gamboost(fm, data = data, offset = fitted(offsetmod),
  control = boost_control(mstop = 250, nu = 0.2))
options(contrasts = c("contr.treatment", "contr.poly"))
  save(offsetmod, mod, file = "pathway_model.Rda")
} else {
  load("pathway_model.Rda")
}


###################################################
### code chunk number 36: cvrisk_mod_code (eval = FALSE)
###################################################
## ## to speed up computation on linux systems set number of
## ## cores in cvrisk: e.g. mc.cores = 25
## cvr <- cvrisk(mod, grid = 1:2000)


###################################################
### code chunk number 37: cvrisk_mod
###################################################
if (!file.exists("cvrisk_mod.Rda")) {
## to speed up computation on linux systems set number of
## cores in cvrisk: e.g. mc.cores = 25
cvr <- cvrisk(mod, grid = 1:2000)
  save(cvr, file = "cvrisk_mod.Rda")
} else {
  load("cvrisk_mod.Rda")
}


###################################################
### code chunk number 38: cvrisk_mod_shown (eval = FALSE)
###################################################
## ## optimal number of iterations
## mstop(cvr)
## ## plot the cross-validation results
## plot(cvr)
## ## finally set model iterations to this number
## mstop(mod) <- mstop(cvr)


###################################################
### code chunk number 39: selection_frac
###################################################
prob <- round(length(coef(mod)) / length(coef(mod, which = "")), 3) * 100


###################################################
### code chunk number 40: set_mstop
###################################################
## set mstop to 250 again to increase speed of stability selection
mstop(mod) <- 250


###################################################
### code chunk number 41: stabsel_code (eval = FALSE)
###################################################
## ## use stability selection to extract important pathways / substrate classes
## ## to speed up computation on linux systems set number of
## ## cores in stabsel: e.g. mc.cores = 25
## stab <- stabsel(mod, cutoff = 0.75, PFER = 1.5, assumption = "unimodal")


###################################################
### code chunk number 42: stabsel
###################################################
if (!file.exists("stabsel.Rda")) {
## use stability selection to extract important pathways / substrate classes
## to speed up computation on linux systems set number of
## cores in stabsel: e.g. mc.cores = 25
stab <- stabsel(mod, cutoff = 0.75, PFER = 1.5, assumption = "unimodal")
  save(stab, file = "stabsel.Rda")
} else {
  load("stabsel.Rda")
}


###################################################
### code chunk number 43: model_pathways_code (eval = FALSE)
###################################################
## ## extract variable names
## nms <- gsub(", ", " by ", variable.names(mod))
## ## adapt plotting settings
## old.par <- par(mar = c(4, 0.1, 0.1, 11.5), mfrow = c(1, 2))
## plot(stab, type = "paths", main = "", labels = nms)
## par(mar = c(4, 11.5, 0.1, 0.1))
## plot(stab, type = "maxsel", main = "", np = 20, labels = nms)
## par(old.par) # reset plotting settings


###################################################
### code chunk number 44: model_pathways
###################################################
## extract variable names
nms <- gsub(", ", " by ", variable.names(mod))
## adapt plotting settings
old.par <- par(mar = c(4, 0.1, 0.1, 11.5), mfrow = c(1, 2))
plot(stab, type = "paths", main = "", labels = nms)
par(mar = c(4, 11.5, 0.1, 0.1))
plot(stab, type = "maxsel", main = "", np = 20, labels = nms)
par(old.par) # reset plotting settings


###################################################
### code chunk number 45: coefficients_code (eval = FALSE)
###################################################
## mstop(mod) <- mstop(cvr)
## cf <- coef(mod, which = selected(stab))
## ## make cf a vector
## beta <- unlist(cf)
## old.par <- par(mar = c(4, 15, 0.1, 0.1))# adapt plotting settings
## ## plot 2 * beta as this is the difference between the two strains
## plot(2 * sort(beta), 1:length(beta),
##   type = "n", yaxt = "n",
##   ylab = "", xlab = expression(paste("Difference between strains (",
##     2 * hat(beta), ")")))
## abline(h = 1:length(beta), lty = "dotted", col = "grey")
## points(2 * sort(beta), 1:length(beta), pch = 19)
## nms <- gsub(", ", " by ", variable.names(mod, which = selected(stab)))
## axis(2, at = 1:length(beta), nms[order(beta)], las = 2)
## abline(v = 0, col = "grey")
## par(old.par) # reset plotting settings


###################################################
### code chunk number 46: coefficients
###################################################
mstop(mod) <- mstop(cvr)
cf <- coef(mod, which = selected(stab))
## make cf a vector
beta <- unlist(cf)
old.par <- par(mar = c(4, 15, 0.1, 0.1))# adapt plotting settings
## plot 2 * beta as this is the difference between the two strains
plot(2 * sort(beta), 1:length(beta),
  type = "n", yaxt = "n",
  ylab = "", xlab = expression(paste("Difference between strains (",
    2 * hat(beta), ")")))
abline(h = 1:length(beta), lty = "dotted", col = "grey")
points(2 * sort(beta), 1:length(beta), pch = 19)
nms <- gsub(", ", " by ", variable.names(mod, which = selected(stab)))
axis(2, at = 1:length(beta), nms[order(beta)], las = 2)
abline(v = 0, col = "grey")
par(old.par) # reset plotting settings


###################################################
### code chunk number 47: e-coli-comp-full
###################################################
coli.comp <- opm_mcp(coli.sub, output = "mcp",
  model = ~ J(Well + Strain), linfct = c(Pairs = 1))
coli.anno <- annotated(coli.comp, how = "data.frame")
coli.anno <- coli.anno[complete.cases(coli.anno), ]


###################################################
### code chunk number 48: prep_mcp_dat
###################################################
carb.pos <- rownames(coli.anno)[coli.anno$Carbohydrates == TRUE]
noncarb.pos <- rownames(coli.anno)[coli.anno$Carbohydrates == FALSE]
carb.pos <- find_positions(carb.pos, plate_type(coli.sub))
noncarb.pos <- find_positions(noncarb.pos, plate_type(coli.sub))


###################################################
### code chunk number 49: carb-comp-contr-mat
###################################################
contr.carb <- opm_mcp(coli.sub[, , carb.pos], linfct = c(Dunnett = 1),
  model = ~ J(Well + Strain), output = "contrast")[[1]]
contr.noncarb <- opm_mcp(coli.sub[, , noncarb.pos], linfct = c(Dunnett = 1),
  model = ~ J(Well + Strain), output = "contrast")[[1]]


###################################################
### code chunk number 50: carb-comp-contr-finish
###################################################
contr <- rbind(
  "Carbohydrates/DSM18039 - Carbohydrates/DSM30083T" = c(
    rep(c(2 / ncol(contr.carb), -2 / ncol(contr.carb)),
      times = ncol(contr.carb) / 2),
    rep(0, times = ncol(contr.noncarb))
  ),
  "non-Carb./DSM18039 - non-Carb./DSM30083T" = c(
    rep(0, times = ncol(contr.carb)),
    rep(c(2 / ncol(contr.noncarb), -2 / ncol(contr.noncarb)),
      times = ncol(contr.noncarb) / 2)
  )
)
colnames(contr) <- c(colnames(contr.carb), colnames(contr.noncarb))


###################################################
### code chunk number 51: carb-comp-test
###################################################
carb.test <- opm_mcp(coli.sub[, , c(carb.pos, noncarb.pos)], linfct = contr,
  m.type = "lm", model = ~ J(Well + Strain))


###################################################
### code chunk number 52: carb-comp-test-figure
###################################################
old.par <- par(mar = c(3, 25, 4, 3)) # adapt plotting settings
plot(carb.test, xlim = c(-190, 20))
par(old.par) # reset plotting settings


