### R code from vignette source 'opm-tutorial.Rnw'

###################################################
### code chunk number 1: chunk-setup
###################################################
options(width = 90)
options(prompt = "R> ")
options(continue = "   ")
options(useFancyQuotes = FALSE)
library("methods")
if ("package:opm" %in% search())
  detach("package:opm", unload = TRUE)


###################################################
### code chunk number 2: load-library
###################################################
library("opm")


###################################################
### code chunk number 3: example-data
###################################################
vaas_1
vaas_4


###################################################
### code chunk number 4: big-example-data
###################################################
data(vaas_et_al, package = "opmdata")


###################################################
### code chunk number 5: opm-files
###################################################
files <- opm_files("testdata")
files


###################################################
### code chunk number 6: read-opm
###################################################
example.opm <- read_opm(names = files, convert = "try",
  include = "*Example_?.CSV.xz")
summary(example.opm)


###################################################
### code chunk number 7: read-opm-lims
###################################################
example.lims <- read_opm(names = files, convert = "try",
  include = "*Example_LIMS_*.EXL.xz")
summary(example.lims)
rm(example.lims)


###################################################
### code chunk number 8: read-many-plates
###################################################
many.plates <- read_opm(names = files, exclude = "*Multiple*")
summary(many.plates)
summary(many.plates$PM01)
rm(many.plates) # tidy up


###################################################
### code chunk number 9: batch-opm-demo
###################################################
batch_opm(files, include = "*Example_?.CSV.xz",
  aggr.args = list(boot = 100, method = "opm-fast"),
  outdir = ".", demo = TRUE)


###################################################
### code chunk number 10: batch-opm-real
###################################################
batch.result <- batch_opm(files, include = "*Example_?.CSV.xz",
  aggr.args = list(boot = 100, method = "opm-fast"),
  outdir = ".")


###################################################
### code chunk number 11: opm-scripts
###################################################
opm_files("scripts")


###################################################
### code chunk number 12: collect-template
###################################################
metadata.example <- collect_template(files, include = "*Example_?.CSV.xz")


###################################################
### code chunk number 13: collect-template-with-file
###################################################
collect_template(files, include = "*Example_?.CSV.xz",
  outfile = "example_metadata.CSV")


###################################################
### code chunk number 14: to-metadata-read
###################################################
metadata.example <- to_metadata("example_metadata.CSV")


###################################################
### code chunk number 15: add-metadata-column
###################################################
metadata.example$Colour <- c("blue", "red", "yellow")
metadata.example$Integer <- c(10L, 20L, 30L)


###################################################
### code chunk number 16: include-metadata-from-data-frame
###################################################
example.opm <- include_metadata(example.opm, md = metadata.example)


###################################################
### code chunk number 17: get-metadata
###################################################
metadata(example.opm)


###################################################
### code chunk number 18: tidy-metadata-file-up
###################################################
unlink("example_metadata.CSV")


###################################################
### code chunk number 19: to-metadata-convert
###################################################
metadata(example.opm)
metadata(example.opm) <- to_metadata(csv_data(example.opm))[,
  c("Strain Name", "Sample Number")]
metadata(example.opm)


###################################################
### code chunk number 20: to-metadata-convert-easy (eval = FALSE)
###################################################
## ## do not enter this code, as we already have copied the CSV data
## metadata(example.opm) <- TRUE # set selected CSV metainformation
## metadata(example.opm) <- FALSE # remove it again


###################################################
### code chunk number 21: map-metadata-with-formula
###################################################
metadata(example.opm)
metadata(map_metadata(example.opm, Organism ~ `Sample Number`))


###################################################
### code chunk number 22: clean-metadata-with-formula
###################################################
metadata(map_metadata(example.opm,
  Organism + `Sample Number` ~ list(`Sample Number`, NULL)))


###################################################
### code chunk number 23: OPMS-clean-metadata-with-formula
###################################################
metadata(example.opm) <- Organism + `Sample Number` ~
  list(`Sample Number`, NULL)
metadata(example.opm)


###################################################
### code chunk number 24: metadata-expression
###################################################
metadata(example.opm) <- to_metadata(csv_data(example.opm))[,
  c("Strain Name", "Sample Number")] # reset
metadata(example.opm)
metadata(example.opm) <- expression(Organism <- `Sample Number`,
  rm(`Sample Number`))
metadata(example.opm)


###################################################
### code chunk number 25: empty-metadata
###################################################
metadata(example.opm, "Organism") <- NULL
metadata(example.opm)
metadata(example.opm) <- list()
metadata(example.opm)


###################################################
### code chunk number 26: assign-metadata
###################################################
metadata(example.opm[2]) <- list(Organism = "Elephas maximus",
  Size = "3 meters")
metadata(example.opm)
metadata(example.opm[2]) <- list()
metadata(example.opm)


###################################################
### code chunk number 27: map-included-metadata
###################################################
example.opm <- include_metadata(example.opm, md = metadata.example)
metadata(example.opm)
example.opm <- map_metadata(example.opm, Character ~ as.character(Integer))
metadata(example.opm)
example.opm <- map_metadata(example.opm, `Times 10` ~ (Integer * 10))
metadata(example.opm)


###################################################
### code chunk number 28: metadata-chars
###################################################
example.opm <- include_metadata(example.opm, md = metadata.example)
md.map <- metadata_chars(example.opm, values = FALSE)
md.map


###################################################
### code chunk number 29: map-metadata-names-with-vector
###################################################
md.map["Colour"] <- "Colony colour"
example.opm <- map_metadata(example.opm, md.map, values = FALSE)
metadata(example.opm)


###################################################
### code chunk number 30: map-metadata-values-with-vector
###################################################
md.map <- metadata_chars(example.opm, values = TRUE)
md.map
md.map["red"] <- "green"
example.opm <- map_metadata(example.opm, md.map, values = TRUE)
metadata(example.opm)


###################################################
### code chunk number 31: merge-metadata-by-hand
###################################################
# not recommended
metadata.example$Colour.Position <-
  as.character(interaction(metadata.example$Colour,
    metadata.example$Position, drop = TRUE))


###################################################
### code chunk number 32: merge-metadata-with-formula
###################################################
metadata(example.opm) <- Col.Int ~ paste(`Colony colour`, Integer, sep = ".")
metadata(example.opm)


###################################################
### code chunk number 33: assign-id-to-metadata (eval = FALSE)
###################################################
## ## do not do this unless you need it
## metadata(example.opm) <- "ID"


###################################################
### code chunk number 34: load-example-data
###################################################
data(vaas_1)
vaas_1


###################################################
### code chunk number 35: aggregation
###################################################
vaas_1.reaggr <- do_aggr(vaas_1, boot = 100, method = "opm-fast")


###################################################
### code chunk number 36: aggregation-settings
###################################################
aggr_settings(vaas_1)
aggr_settings(vaas_1.reaggr)


###################################################
### code chunk number 37: aggregated-data
###################################################
summary(aggregated(vaas_1))
summary(aggregated(vaas_1.reaggr))


###################################################
### code chunk number 38: smoothing-splines
###################################################
op <- set_spline_options(type = "smooth.spline") # not recommended
vaas_1.aggr2 <- do_aggr(vaas_1, method = "spline", options = op)


###################################################
### code chunk number 39: data-vaas-small
###################################################
data("vaas_et_al", package = "opmdata")
vaas.small <- vaas_et_al[, , c("A01", "G11", "H11")]
dim(vaas.small)


###################################################
### code chunk number 40: query-metadata-k-one-key
###################################################
"Experiment" %k% vaas_et_al
vaas_et_al %k% "Experiment" # equivalent
vaas_et_al %k% ~ Experiment # equivalent
(~ Experiment) %k% vaas_et_al # equivalent, parentheses needed


###################################################
### code chunk number 41: query-metadata-k-two-keys
###################################################
c("Experiment", "Species") %k% vaas_et_al
vaas_et_al %k% ~ c(Experiment, Species) # equivalent


###################################################
### code chunk number 42: query-metadata-q-keys-with-values
###################################################
c(Experiment = "First replicate",
  Species = "Escherichia coli") %q% vaas_et_al
vaas_et_al %q% ~ Experiment == "First replicate" &
  Species == "Escherichia coli"


###################################################
### code chunk number 43: query-metadata-q-alternative-values
###################################################
list(Species = c("Escherichia coli", "Bacillus subtilis")) %q% vaas_et_al
vaas_et_al %q% ~ Species %in% c("Escherichia coli", "Bacillus subtilis")


###################################################
### code chunk number 44: query-metadata-and-select
###################################################
vaas.e.coli.1 <- vaas_et_al[c(Experiment = "First replicate",
  Species = "Escherichia coli")]
## this is the short notation for:
#vaas.e.coli.1 <- vaas_et_al[c(Experiment = "First replicate",
#  Species = "Escherichia coli") %q% vaas_et_al]
summary(vaas.e.coli.1)
rm(vaas.e.coli.1) # tidy up


###################################################
### code chunk number 45: check-metadata-key
###################################################
metadata_chars(vaas_et_al, values = FALSE)


###################################################
### code chunk number 46: check-metadata-values
###################################################
metadata(vaas_et_al, "Species")


###################################################
### code chunk number 47: subset
###################################################
vaas.1.6 <- subset(vaas_et_al,
  query = list(Experiment = "First replicate", 'Plate number' = 6))
summary(vaas.1.6)


###################################################
### code chunk number 48: extract-simple
###################################################
vaas.mu <- extract(vaas_et_al, dataframe = TRUE,
  as.labels = NULL, subset = "mu")


###################################################
### code chunk number 49: extract-complex
###################################################
vaas.mu <- extract(vaas_et_al, dataframe = TRUE,
  as.labels = list("Experiment", "Number of sample time point",
    "Plate number", "Slot", "Species", "Strain", "Time point in min"),
  subset = "mu")


###################################################
### code chunk number 50: extract-with-joining
###################################################
vaas.mu <- extract(vaas_et_al, dataframe = TRUE,
  as.labels = ~ J(Species, Strain), subset = "mu")


###################################################
### code chunk number 51: xy-plot-figure-code (eval = FALSE)
###################################################
## xy_plot(vaas.1.6[, , 1:24], main = "E. coli vs. P. aeruginosa",
##   include = list("Species", "Strain"))


###################################################
### code chunk number 52: xy-plot-figure
###################################################
xy_plot(vaas.1.6[, , 1:24], main = "E. coli vs. P. aeruginosa",
  include = list("Species", "Strain"))


###################################################
### code chunk number 53: xy-plot-partial-figure-code (eval = FALSE)
###################################################
## xy_plot(vaas.1.6[, , c("D01", "D02", "C10", "C11")], neg.ctrl = NULL,
##   main = "E. coli vs. P. aeruginosa", include = list("Species", "Strain"))


###################################################
### code chunk number 54: xy-plot-partial-figure
###################################################
xy_plot(vaas.1.6[, , c("D01", "D02", "C10", "C11")], neg.ctrl = NULL,
  main = "E. coli vs. P. aeruginosa", include = list("Species", "Strain"))


###################################################
### code chunk number 55: level-plot-figure-code (eval = FALSE)
###################################################
## level_plot(vaas.1.6[, , 1:24], main = "E. coli vs. P. aeruginosa",
##   include = list("Species", "Strain"))


###################################################
### code chunk number 56: level-plot-figure
###################################################
level_plot(vaas.1.6[, , 1:24], main = "E. coli vs. P. aeruginosa",
  include = list("Species", "Strain"))


###################################################
### code chunk number 57: para-plot-figure-code (eval = FALSE)
###################################################
## parallel_plot(vaas_4[, , 1:10], data = ~ J(Species, Strain))


###################################################
### code chunk number 58: para-plot-figure
###################################################
parallel_plot(vaas_4[, , 1:10], data = ~ J(Species, Strain))


###################################################
### code chunk number 59: para-plot-advanced-figure-code (eval = FALSE)
###################################################
## parallel_plot(vaas_4[, , 1:10], data = ~ J(Species, Strain),
##   panel.var = "Species", groups = "Strain")


###################################################
### code chunk number 60: para-plot-advanced
###################################################
parallel_plot(vaas_4[, , 1:10], data = ~ J(Species, Strain),
  panel.var = "Species", groups = "Strain")


###################################################
### code chunk number 61: radial-plot-figure-code (eval = FALSE)
###################################################
## radial_plot(vaas_4[, , c(1:5, 10)], as.labels = list("Species", "Strain"),
##   x = 150, y = 200)


###################################################
### code chunk number 62: radial-plot-figure
###################################################
radial_plot(vaas_4[, , c(1:5, 10)], as.labels = list("Species", "Strain"),
  x = 150, y = 200)


###################################################
### code chunk number 63: radial-plot-symbols-figure-code (eval = FALSE)
###################################################
## radial_plot(vaas_4[, , c(1:5, 10)], as.labels = list("Species", "Strain"),
##   main = "Test", x = -550, y = -50, rp.type = "s",
##   point.symbols = 15, show.centroid = TRUE)


###################################################
### code chunk number 64: radial-plot-symbols-figure
###################################################
radial_plot(vaas_4[, , c(1:5, 10)], as.labels = list("Species", "Strain"),
  main = "Test", x = -550, y = -50, rp.type = "s",
  point.symbols = 15, show.centroid = TRUE)


###################################################
### code chunk number 65: ci-plot-figure-code (eval = FALSE)
###################################################
## ci_plot.legend <- ci_plot(vaas.1.6[, , c("A01", "A02", "A03")],
##   as.labels = list("Species", "Strain"), subset = "A",
##   legend.field = NULL, x = 170, y = 3)


###################################################
### code chunk number 66: ci-plot-figure
###################################################
ci_plot.legend <- ci_plot(vaas.1.6[, , c("A01", "A02", "A03")],
  as.labels = list("Species", "Strain"), subset = "A",
  legend.field = NULL, x = 170, y = 3)


###################################################
### code chunk number 67: extract-dataframe
###################################################
x <- extract(vaas_et_al, as.labels = list("Species", "Strain"),
  dataframe = TRUE)


###################################################
### code chunk number 68: ci-plot-of-groups-figure-code (eval = FALSE)
###################################################
## # without normalisation
## ci_plot(extract(x, as.groups = TRUE, norm.per = "none")[, c(1:7, 13)],
##   legend.field = NULL, x = 350, y = 0)


###################################################
### code chunk number 69: ci-plot-of-groups-figure
###################################################
# without normalisation
ci_plot(extract(x, as.groups = TRUE, norm.per = "none")[, c(1:7, 13)],
  legend.field = NULL, x = 350, y = 0)


###################################################
### code chunk number 70: ci-plot-normalisation (eval = FALSE)
###################################################
## ci_plot(extract(x, as.groups = TRUE, norm.per = "row")[, c(1:7, 13)],
##   legend.field = NULL, x = 150, y = 0)
## ci_plot(extract(x, as.groups = TRUE, norm.per = "column")[, c(1:7, 13)],
##   legend.field = NULL, x = 150, y = 0)


###################################################
### code chunk number 71: ci-plot-norm-well-A10-figure-code (eval = FALSE)
###################################################
## ci_plot(extract(x, as.groups = TRUE, norm.per = "row",
##   norm.by = 10, subtract = TRUE)[, c(1:7, 13)],
##   legend.field = NULL, x = 0, y = 0)


###################################################
### code chunk number 72: ci-plot-norm-well-A10-figure
###################################################
ci_plot(extract(x, as.groups = TRUE, norm.per = "row",
  norm.by = 10, subtract = TRUE)[, c(1:7, 13)],
  legend.field = NULL, x = 0, y = 0)


###################################################
### code chunk number 73: heat-map-figure-code (eval = FALSE)
###################################################
## vaas.1.6.A <- heat_map(vaas.1.6, as.labels = "Strain",
##   as.groups = "Species")


###################################################
### code chunk number 74: heat-map-figure
###################################################
vaas.1.6.A <- heat_map(vaas.1.6, as.labels = "Strain",
  as.groups = "Species")


###################################################
### code chunk number 75: data-vaas-subset
###################################################
vaas.G06 <- subset(vaas_et_al[, , "G06"],
  list(Experiment = "First replicate"))


###################################################
### code chunk number 76: plot-vaas-subset-figure-code (eval = FALSE)
###################################################
## xy_plot(vaas.G06, include = ~ Strain, neg.ctrl = FALSE, space = "right")


###################################################
### code chunk number 77: plot-vaas-subset-figure
###################################################
xy_plot(vaas.G06, include = ~ Strain, neg.ctrl = FALSE, space = "right")


###################################################
### code chunk number 78: mcp-data
###################################################
head(x <- opm_mcp(vaas.G06, model = ~ Strain, m.type = "aov",
  linfct = c(Tukey = 1), output = "data", full = FALSE))


###################################################
### code chunk number 79: mcp-principle
###################################################
vaas.G06.mcp <- opm_mcp(vaas.G06, model = ~ Strain, m.type = "aov",
  linfct = c(Tukey = 1))


###################################################
### code chunk number 80: mcp-output-model (eval = FALSE)
###################################################
## opm_mcp(vaas.G06, model = ~ Strain, m.type = "aov",
##   linfct = c(Tukey = 1), output = "model")


###################################################
### code chunk number 81: mcp-output-linfct (eval = FALSE)
###################################################
## opm_mcp(vaas.G06, model = ~ Strain, m.type = "aov",
##   linfct = c(Tukey = 1), output = "linfct")


###################################################
### code chunk number 82: plot-mcp-linfct-figure-code (eval = FALSE)
###################################################
## old.mar <- par(mar = c(3, 15, 3, 2)) # adapt margins in the plot
## plot(vaas.G06.mcp)
## par(old.mar) # reset to default plotting settings


###################################################
### code chunk number 83: plot-mcp-linfct-figure
###################################################
old.mar <- par(mar = c(3, 15, 3, 2)) # adapt margins in the plot
plot(vaas.G06.mcp)
par(old.mar) # reset to default plotting settings


###################################################
### code chunk number 84: summary-mcp-Tukey
###################################################
mcp.summary <- summary(vaas.G06.mcp)
mcp.summary$model$call <- NULL # avoid some unnecessary output
mcp.summary


###################################################
### code chunk number 85: vaas-e-coli
###################################################
vaas.e.coli <- subset(vaas_et_al,
  list(Experiment = "First replicate", Strain = "DSM30083T"))


###################################################
### code chunk number 86: vaas-e-coli-mcp
###################################################
opm_mcp(vaas.e.coli[, , 1:10], output = "mcp", model = ~ Well,
  linfct = c(`Dunnett_A01 (Negative Control)` = 1))


###################################################
### code chunk number 87: vaas-e-coli-mcp-Dunnett-special
###################################################
mcp.A03 <- opm_mcp(vaas.e.coli[, , 1:10], output = "mcp", model = ~ Well,
  linfct = c(Dunnett_A03 = 1), full = FALSE)
mcp.summary <- summary(mcp.A03)
mcp.summary$model$call <- NULL # avoid some unnecessary output
mcp.summary


###################################################
### code chunk number 88: mcp-linfct-pairs-showdata
###################################################
opm_mcp(vaas_4[, , c(1:3, 86)], model = ~ J(Well, Species), m.type = "aov",
  linfct = c(Pairs = 1), output = "data", full = FALSE)


###################################################
### code chunk number 89: mcp-linfct-pairs-runanalysis
###################################################
y <- opm_mcp(vaas_4[, , c(1:3, 86)], model = ~ J(Well, Species),
  m.type = "aov", linfct = c(Pairs.Well = 1), full = FALSE)


###################################################
### code chunk number 90: summary-mcp-pairs
###################################################
mcp.summary <- summary(y)
mcp.summary$model$call <- NULL # avoid some unnecessary output
mcp.summary


###################################################
### code chunk number 91: mcp-linfct-pairs-species
###################################################
y <- opm_mcp(vaas_4[, , c(1:3, 86)], model = ~ J(Well, Species), m.type = "aov",
  linfct = c(Pairs.Species = 1), full = FALSE)
mcp.summary <- summary(y)
mcp.summary$model$call <- NULL # avoid some unnecessary output
mcp.summary


###################################################
### code chunk number 92: contrMat
###################################################
library("multcomp") # now needed
n <- c(10, 20, 30, 40)
names(n) <- paste0("group", 1:4)
contrMat(n, type = "Tukey")


###################################################
### code chunk number 93: mcp-summary-linfct
###################################################
summary(vaas.G06.mcp)$linfct


###################################################
### code chunk number 94: contrast-matrix
###################################################
contr <- opm_mcp(vaas_4[, , 1:4], model = ~ Well, linfct = c(Tukey = 1),
  output = "contrast", full = FALSE)
contr


###################################################
### code chunk number 95: mcp-wells
###################################################
vaas4.mcp <- opm_mcp(vaas_4[, , 1:4], model = ~ Well, m.type = "lm",
  linfct = contr$Well[c(1:3, 6), ], full = FALSE)


###################################################
### code chunk number 96: summary-mcp-wells
###################################################
summary(vaas4.mcp)$linfct


###################################################
### code chunk number 97: vaas4-mcp-figure
###################################################
old.mar <- par(mar = c(3, 20, 3, 2)) # adapt plotting settings
plot(vaas4.mcp)
par(old.mar) # reset plotting settings


###################################################
### code chunk number 98: model-mcp
###################################################
vaas4.mcp <- opm_mcp(vaas_4, model = ~ Species, m.type = "lm",
  linfct = mcp(Species = "Dunnett"))


###################################################
### code chunk number 99: analyse-G06-data
###################################################
vaas.G06 <- vaas_et_al[, , "G06"]
vaas.G06.mcp <- opm_mcp(vaas.G06, model = ~ J(Strain, Experiment),
  linfct = c(Dunnett = 1))


###################################################
### code chunk number 100: vaas-G06-mcp-figure
###################################################
old.mar <- par(mar = c(3, 22, 3, 1)) # adapt plotting settings
plot(vaas.G06.mcp)
par(old.mar) # reset plotting settings


###################################################
### code chunk number 101: mcp-G06-output-contrast
###################################################
contr <- opm_mcp(vaas.G06, model = ~ J(Strain, Experiment),
  linfct = c(Dunnett = 1), output = "contrast")$Strain.Experiment
colnames(contr)


###################################################
### code chunk number 102: mcp-G06-user-def-contrasts
###################################################
contr <- contr[1:3, ] # keeps the column names
rownames(contr) <- c(
  "First repl. - Second repl.",
  "First repl. - Time series",
  "DSM 1707 #1 - Second repl."
)
contr[1, ] <- c(1 / 4, -1 / 4, 1 / 4, -1 / 4, 1 / 4, -1 / 4, 0, 1 / 4, -1 / 4)
contr[2, ] <- c(1 / 4, 0, 1 / 4, 0, 1 / 4, 0, -1, 1 / 4, 0)
contr[3, ] <- c(0, -1 / 4, 1, -1 / 4, 0, -1 / 4, 0, 0, -1 / 4)
contr
vaas6.mcp <- opm_mcp(vaas.G06, model = ~ J(Strain, Experiment), m.type = "lm",
  linfct = mcp(Strain.Experiment = contr))


###################################################
### code chunk number 103: mcp-G06-user-def-contrasts-figure
###################################################
old.mar <- par(mar = c(3, 12, 3, 2)) # adapt plotting settings
plot(vaas6.mcp)
par(old.mar) # reset plotting settings


###################################################
### code chunk number 104: discretise
###################################################
vaas.repl <- subset(vaas_et_al,
  query = list(Experiment = c("First replicate", "Second replicate")))
vaas.repl <- do_disc(vaas.repl)


###################################################
### code chunk number 105: list-discretise
###################################################
listing(vaas.repl, as.groups = NULL)
listing(vaas.repl, as.groups = list("Species"))


###################################################
### code chunk number 106: phylo-data-text
###################################################
phylo_data(listing(vaas.repl, as.groups = NULL))
phylo_data(listing(vaas.repl, as.groups = list("Species")))


###################################################
### code chunk number 107: phylo-data-html
###################################################
phylo_data(listing(vaas.repl, as.groups = NULL, html = TRUE))
phylo_data(listing(vaas.repl, as.groups = list("Species"), html = TRUE))


###################################################
### code chunk number 108: opm-css-file
###################################################
opm_opt(css.file = opm_files("css")[[1]])


###################################################
### code chunk number 109: opm-html-file
###################################################
vaas.html <- phylo_data(vaas.repl, format = "html",
  as.labels = list("Species", "Strain"), outfile = "vaas.html")


###################################################
### code chunk number 110: wd-css-file
###################################################
vaas.html <- phylo_data(vaas.repl, format = "html",
  html.args = html_args(embed.css = TRUE),
  as.labels = list("Species", "Strain"), outfile = "vaas.html")
## the alternative: copying the file into the working directory
# file.copy(opm_files("css")[[1]], "opm_styles.css", overwrite = TRUE)
# opm_opt(css.file = "opm_styles.css")


###################################################
### code chunk number 111: prep-disc-data
###################################################
vaas.repl <- subset(vaas_et_al,
  query = list(Experiment = c("First replicate", "Second replicate")))
vaas.repl <- extract(vaas.repl,
  as.labels = list("Species", "Strain", "Experiment", "Plate number"))


###################################################
### code chunk number 112: disc-data-32-states
###################################################
vaas.repl.disc <- discrete(vaas.repl, range = c(0, 400))


###################################################
### code chunk number 113: phylo-data-epf
###################################################
phylo_data(vaas.repl.disc, outfile = "example_replicates.epf")


###################################################
### code chunk number 114: disc-data-binary
###################################################
vaas.repl.disc <- discrete(vaas.repl, range = c(120.2, 236.6), gap = TRUE)


###################################################
### code chunk number 115: disc-data-tertiary
###################################################
vaas.repl.disc <- discrete(vaas.repl, range = c(120.2, 236.6),
  gap = TRUE, middle.na = FALSE)


